package controlador;

import componentes.ResultadoTransaccion;
import static componentes.ResultadoTransaccion.ErrorType.*;
import modelo.Cuenta;
import modelo.Datos;
import org.apache.commons.validator.routines.IBANValidator;
import vista.VistaTransacciones;

/**
 *
 * @author jonas.schiefner
 */
public class ControladorTransacciones {
    private static final IBANValidator validator = new IBANValidator();
    
    private VistaTransacciones vista;
    private Cuenta cuenta;
    
    public ControladorTransacciones(VistaTransacciones vista, Cuenta cuenta) {
        this.vista = vista;
        this.cuenta = cuenta;
    }
        
    public ResultadoTransaccion realizarTransaccion(String nombreTitular, String iban, String saldoString, String concepto) {
        // validar nombre titular
        if (nombreTitular.isBlank()) {
            return new ResultadoTransaccion(FALTA_NOMBRE_TITULAR);
        }
        
        // validar iban
        if (!validator.isValid(iban)) {
            return new ResultadoTransaccion(IBAN_INVALIDO);
        }
        
        Cuenta destino = Datos.findCuentaConIban(iban);
        if (destino == null) {
            return new ResultadoTransaccion(FALTA_CUENTA);
        }
        if (destino == cuenta) {
            return new ResultadoTransaccion(MISMO_CUENTA);
        }
        
        // validar concepto
        if (concepto.isBlank()) {
            return new ResultadoTransaccion(FALTA_CONCEPTO);
        }
        
        // validar saldo
        if (saldoString.isBlank()) {
            return new ResultadoTransaccion(FALTA_SALDO);
        }
        int saldo;
        try {
            saldo = Integer.parseInt(saldoString);
        } catch (NumberFormatException e) {
            return new ResultadoTransaccion(SALDO_INVALIDO);
        }
        if (saldo <= 0) {
            return new ResultadoTransaccion(SALDO_INVALIDO);
        }
        if (saldo > cuenta.getSaldo()) {
            return new ResultadoTransaccion(NO_CUBRE_SALDO, saldo, cuenta.getSaldo());
        }
        
        // si todo esta bien, realizamos la transaccion
        cuenta.inicializarTransaccion(destino, saldo, nombreTitular, concepto);
        
        // para indicar que se ha realizada la transaccion el string debe empezar con "¡Hecho!"
        return new ResultadoTransaccion(saldo, cuenta.getSaldo());
    }
}
